<?php


/**
 * Image Editor で使用する共通メソッドを定義する
 * 
 */


// Do not allow direct access to the file.
if( ! defined( 'ABSPATH' ) ) {
	exit;
}




// 画像エディタに追加したClassを加える
add_filter( 'wp_image_editors', function( $image_editors ) {

	if( ! is_array( $image_editors ) ) {
		return $image_editors;
	}

	$new_editors = $image_editors;

	if( StillBE_WP_Image_Editor_Imagick::test() && StillBE_WP_Image_Editor_Imagick::supports_mime_type( 'image/webp' ) ) {
		@ array_unshift( $new_editors, 'StillBE_WP_Image_Editor_Imagick', 'StillBE_WP_Image_Editor_GD' );
	} elseif( StillBE_WP_Image_Editor_GD::test() && StillBE_WP_Image_Editor_GD::supports_mime_type( 'image/webp' ) ) {
		@ array_unshift( $new_editors, 'StillBE_WP_Image_Editor_GD', 'StillBE_WP_Image_Editor_Imagick' );
	} else {
		@ array_unshift( $new_editors, 'StillBE_WP_Image_Editor_Imagick', 'StillBE_WP_Image_Editor_GD' );
	}

	return count( $new_editors ) > count( $image_editors ) ? $new_editors : $image_editors;

} );




// アップロード時、オリジナルサイズの画像のWebPを生成する
add_filter( 'wp_handle_upload', function( $upload ) {

	// 画像 (image/*) 以外は何もしない
	// Mime Type は wp_get_image_editor 関数で行われる

	// Image Editor
	$editor = wp_get_image_editor( $upload['file'] );
	if( is_wp_error( $editor ) ) {
		// This image cannot be edited.
		return $upload;
	}

	// Deny a Multibyte Filename
	$new_filename = $editor->generate_safe_filename( $upload['file'], '', true );
	if( $upload['file'] !== $new_filename ) {
		require_once( ABSPATH. 'wp-admin/includes/file.php' );
		global $wp_filesystem;
		if( WP_Filesystem() ) {
			$file_system = &$wp_filesystem;
		} else {
		//	wp_die( __( 'WP Filesystem is not available.', 'stillbedomain' ) );
		}
		$copied = $file_system->copy( $upload['file'], $new_filename, false );
		if( $copied ) {
			unlink( $upload['file'] );
			$upload['file'] = $new_filename;
			$editor = wp_get_image_editor( $upload['file'] );
		}
	}

	// 画像をロード
	$editor->load();

	// WebP 画像を生成
	$editor->make_webp( "{$upload['file']}.webp" );

	// $editor を削除
	$editor = null;
	unset( $editor );

	// フィルターの値は変更せず返す
	return $upload;

} );




// make_subsize メソッドに渡されるサイズデータにサイズ名を追加する
function _stillbe_image_subsizes_size_name( $new_sizes ) {

	if( ! is_array( $new_sizes ) ) {
		return array();
	}

	foreach( $new_sizes as $size => $data ) {
		$new_sizes[ $size ]['size_name'] = $size;
	}

	return $new_sizes;

}
add_filter( 'intermediate_image_sizes_advanced', '_stillbe_image_subsizes_size_name', 9999 );
add_filter( 'wp_get_missing_image_subsizes',     '_stillbe_image_subsizes_size_name', 9999 );




// Mime-Type に WebP を追加する
add_filter( 'mime_types', function( $list ) {

	$new_list = $list;

	if( empty( $new_list['webp'] ) ) {
		$new_list['webp'] = 'image/webp';
	}

	return $new_list;

} );




// 画像削除する時に対応するWebPがあれば一緒に削除する
add_filter( 'wp_delete_file', function( $filename ) {

	if( empty( $filename ) ||
	      ! preg_match( '/^(jpe?g|png|gif)$/i', pathinfo( $filename, PATHINFO_EXTENSION ) ) ) {
		return $filename;
	}

	$_webp = $filename. '.webp';

	if( file_exists( $_webp ) ) {
		@unlink( $_webp );
	}

	return $filename;

} );




// END of the File



