<?php

// Do not allow direct access to the file.
if( ! defined( 'ABSPATH' ) ) {
	exit;
}




// Regenerate Resized Images with Ajax
add_action( 'wp_ajax_sb_iqc_regenerate_images', function(){

	// Nonce Check
	if( ! wp_verify_nonce( $_POST['_nonce'], 'sb-iqc-regenerate-images' ) ) {
		exit( json_encode( array(
			'ok'      => false,
			'id'      => null,
			'message' => __( 'The page has expired. Please reload the page.', 'stillbedomain' ),
		) ) );
	}

	// Getting the attachment ID
	$attachment_id = filter_input( INPUT_POST, 'attachment_id' );
	$attachment_id = absint( $attachment_id ?: 0 );
	if( empty( $attachment_id ) ) {
		exit( json_encode(
			array(
				'ok'      => false,
				'id'      => null,
				'message' => __( 'Attachment ID is not accepted....', 'stillbedomain' ),
			)
		) );
	}

	// Get the Current Metadata
	$old_meta = wp_get_attachment_metadata( $attachment_id );
	if( empty( $old_meta ) || empty( $old_meta['file'] ) ) {
		exit( json_encode(
			array(
				'ok'      => false,
				'id'      => $attachment_id,
				'message' => sprintf( __( 'Attachment ID = %d is not found....', 'stillbedomain' ), $attachment_id ),
			)
		) );
	}

	// Original File Path
	$upload_dir = wp_upload_dir();
	$filename   = $upload_dir['basedir']. '/'. $old_meta['file'];

	// Load Image Editor
	require_once( ABSPATH. 'wp-admin/includes/image.php' );

	// Generate a WebP for Original File
	if( apply_filters( 'stillbe_image_quality_control_enable_webp', STILLBE_IQ_ENABLE_WEBP ) ) {
		$editor = wp_get_image_editor( $filename );
		if( ! is_wp_error( $editor ) ) {
			$editor->load();
			$editor->make_webp( $filename. '.webp' );
			$editor = null;
			unset( $editor );
		}
	}

	// Regenerate Resized Images & Update Metadata
	$new_meta = wp_generate_attachment_metadata( $attachment_id, $filename );

	// Execute it because the reference of the wp_generate_attachment_metadata()
	// function instructs to use it together with the wp_update_attachment_metadata() function.
	// However, since the metadata is updated inside the wp_generate_attachment_metadata() function,
	// the return value of the wp_update_attachment_metadata() function is always (boolern) false
	// (because it tries to update to the same data as the data in the DB).
	// Therefore, the return value of the wp_update_attachment_metadata() function
	// is not used for success / failure judgment.
	wp_update_attachment_metadata( $attachment_id, $new_meta );

	// Result
	$result = ! empty( $new_meta );

	exit( json_encode(
		array(
			'ok'      => $result,
			'id'      => $attachment_id,
			'message' => $result ? __( 'Success!!', 'stillbedomain' ) : __( 'Failed....', 'stillbedomain' ),
			'meta'    => array(
				'old' => $old_meta,
				'new' => $new_meta,
			),
		)
	) );

} );





// END

?>