<?php


/**
 * 設定値を各フィルターに適応する
 * 
 */


// Do not allow direct access to the file.
if( ! defined( 'ABSPATH' ) ) {
	exit;
}




add_action( 'plugins_loaded', function() {

	// Load Settings
	$settings = get_option( StillBE_Img_Quality_Ctrl_Setting::SETTING_NAME, null );

	// No Setting is Loaded
	if( empty( $settings ) ) {
		return;
	}

	// Image Qualitie Levels
	add_filter( 'stillbe_image_quality_default_list', function( $default ) use( $settings ) {
		if( empty( $settings['quality'] ) ) {
			return $default;
		}
		return wp_parse_args( $settings['quality'], $default );
	}, 1 );

	// Toogle Options
	$toggle = $settings['toggle'];

	// No Toogle Settings
	if( empty( $toggle ) ) {
		return;
	}

	// Guarantee a Secure Filename; true
	add_filter( 'stillbe_image_quality_control_convert_safename', function( $is_enable ) use( $toggle ) {
		if( ! isset( $toggle['safe-name'] ) ) {
			return $is_enable;
		}
		return ! empty( $toggle['safe-name'] );
	}, 1 );

	// Autoset Alt from Exif; false
	add_filter( 'stillbe_image_quality_autoset_alt_uploaded_jpeg_exif', function( $is_enable ) use( $toggle ) {
		if( ! isset( $toggle['autoset-alt'] ) ) {
			return $is_enable;
		}
		return ! empty( $toggle['autoset-alt'] );
	}, 1 );

	// Add a Suffix to Indicate Quality Level; false
	add_filter( 'stillbe_image_quality_control_suffix_q_value', function( $is_enable ) use( $toggle ) {
		if( ! isset( $toggle['level-suffix'] ) ) {
			return $is_enable;
		}
		return ! empty( $toggle['level-suffix'] );
	}, 1 );

	// Enable Interlace; true
	add_filter( 'stillbe_image_quality_control_enable_interlace', function( $is_enable ) use( $toggle ) {
		if( ! isset( $toggle['enable-interlace'] ) ) {
			return $is_enable;
		}
		return ! empty( $toggle['enable-interlace'] );
	}, 1 );

	// Enable WebP; true
	add_filter( 'stillbe_image_quality_control_enable_webp', function( $is_enable ) use( $toggle ) {
		if( ! isset( $toggle['enable-webp'] ) ) {
			return $is_enable;
		}
		return ! empty( $toggle['enable-webp'] );
	}, 1 );

	// Enable cwep Liberary; true
	add_filter( 'stillbe_image_quality_control_enable_cwebp_lib', function( $is_enable ) use( $toggle ) {
		if( ! isset( $toggle['enable-cwebp'] ) ) {
			return $is_enable;
		}
		return ! empty( $toggle['enable-cwebp'] );
	}, 1 );

	// Enable Lossless Compression; true
	add_filter( 'stillbe_image_quality_control_enable_webp_lossless_for_png_gif', function( $is_enable ) use( $toggle ) {
		if( ! isset( $toggle['enable-webp-lossless'] ) ) {
			return $is_enable;
		}
		return ! empty( $toggle['enable-webp-lossless'] );
	}, 1 );

	// Enable Near Lossless Compression; false
	add_filter( 'stillbe_image_quality_control_enable_webp_near_lossless', function( $is_enable ) use( $toggle ) {
		if( ! isset( $toggle['enable-webp-near-lossless'] ) ) {
			return $is_enable;
		}
		return ! empty( $toggle['enable-webp-near-lossless'] );
	}, 1 );

} );
