<?php

// Do not allow direct access to the file.
if( ! defined( 'ABSPATH' ) ) {
	exit;
}




class StillBE_Img_Quality_Ctrl_Setting {

	const SETTING_GROUP = STILLBE_IQ_PREFIX. 'setting-group';
	const SETTING_NAME  = STILLBE_IQ_PREFIX. 'setting-option';

	private $settings   = null;
	private $_default   = null;
	private $current    = null;
	private $_editor    = null;


	// Constructer
	public function __construct() {

		// Curernt Settings
		$this->settings = get_option( self::SETTING_NAME, array() );

		// Default Settings
		$this->_default = _stillbe_get_quality_level_array();

		// Current Settings
		// When it is Empty, Use the Plugin's Default Settings
		$this->current  = wp_parse_args(
			$this->settings,
			array(
				'quality' => $this->_default,
				'toggle'  => array(
					'safe-name'                 => true,
					'autoset-alt'               => STILLBE_IQ_AUTOSET_ALT_FROM_EXIF,         // false
					'level-suffix'              => STILLBE_IQ_ENABLE_QUALITY_VALUE_SUFFIX,   // false
					'enable-interlace'          => STILLBE_IQ_ENABLE_INTERLACE,              // true
					'enable-webp'               => STILLBE_IQ_ENABLE_WEBP,                   // true
					'enable-cwebp'              => STILLBE_IQ_ENABLE_CWEBP_LIBRARY,          // true
					'enable-webp-lossless'      => STILLBE_IQ_ENABLE_WEBP_LOSSLESS,          // true
					'enable-webp-near-lossless' => STILLBE_IQ_ENABLE_WEBP_NEAR_LOSSLESS,     // false
				),
			),
		);

		// Image Editor
		if( StillBE_WP_Image_Editor_Imagick::test() ) {
			$this->_editor = 'StillBE_WP_Image_Editor_Imagick';
		} elseif( StillBE_WP_Image_Editor_GD::test() ) {
			$this->_editor = 'StillBE_WP_Image_Editor_GD';
		}

		// Add a Submenu Page
		add_action( 'admin_menu', function() {

		//	add_submenu_page(
		//	Wrapper Function adding submenu page to the Settings main menu.
			add_options_page(
			//	'options-general.php',   // Parent Slug
				__( 'Image Qualities Setting', 'stillbedomain' ),   // Page Title
				__( 'Image Qualities', 'stillbedomain' ),    // Menu Title
				'manage_options',   // Capability
				STILLBE_IQ_PREFIX. 'setting-page',   // Menu Slug
				array( $this, 'render_setting_page' )   // Rederer
			);

		} );

		// Add settings link to plugin actions
		add_filter( 'plugin_action_links', function( $plugin_actions, $plugin_file ) {

			if( basename( STILLBE_IQ_BASE_DIR ). '/stillbe-image-quality-control.php' !== $plugin_file ) {
				return $plugin_actions;
			}

			return array_merge(
				array(
					'sb_iqc_settings' => sprintf(
						__( '<a href="%s">Settings</a>', 'stillbedomain' ),
						esc_url( admin_url( 'options-general.php?page=sb-imgq-setting-page' ) )
					),
				),
				$plugin_actions
			);

		}, 10, 2 );

		// Load CSS / Javascript for Admin
		add_action( 'admin_enqueue_scripts', function() {

			// CSS
			wp_enqueue_style(
				'stillbe-iqc-admin-form-common',
				STILLBE_IQ_BASE_URL. 'asset/admin.css',
				array(),
				@filemtime( STILLBE_IQ_BASE_DIR. '/asset/admin.css' )
			);

			// Javascript
			wp_enqueue_script(
				'stillbe-iqc-admin-form-ajax',
				STILLBE_IQ_BASE_URL.'asset/ajax.js',
				array(),
				@filemtime( STILLBE_IQ_BASE_DIR. '/asset/ajax.js' )
			);

		} );

		// Add Settings
		add_action( 'admin_init', function() {

			// Setting API
			//  * Arg 1 : Group Name using Setting API
			//  * Arg 2 : Saving Key Name in WP Options table (in SQL)
			register_setting( self::SETTING_GROUP, self::SETTING_NAME, array( $this, 'sanitize_setting' ) );

		}, 10 );

		// Add some Setting Sections
		add_action( 'admin_init', function() {

			// * Genral Settings Section
			add_settings_section(
				STILLBE_IQ_PREFIX. 'ss-general',   // Section ID (Slug)
				__( 'General Settings', 'stillbedomain' ),   // Section Title
				array( $this, 'render_sd_general' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page'   // Rendering Page
			);

			// * Advanced Settings Section
			add_settings_section(
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section ID (Slug)
				__( 'Advanced Settings', 'stillbedomain' ),   // Section Title
				array( $this, 'render_sd_advanced' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page'   // Rendering Page
			);

		}, 11 );

		//////////////////////////
		// Add some Setting Fields
		// * General Settings Section
		add_action( 'admin_init', function() {

			// Quality Level
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-quality-level',   // Field ID (Slug)
				__( 'Quality Level of Compression', 'stillbedomain' ),   // Field Label
				array( $this, 'render_quality_level_table' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-general',   // Section
				array()   // Arguments for Renderer Function
			);

			// Default Quality Level
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-default-quality-level',   // Field ID (Slug)
				__( 'Default Quality Level', 'stillbedomain' ),   // Field Label
				array( $this, 'render_quality_level_default' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-general',   // Section
				array()   // Arguments for Renderer Function
			);

			// Quality Level for WebP of the Original Image
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-original-quality-level',   // Field ID (Slug)
				__( 'Quality for webp of the original image', 'stillbedomain' ),   // Field Label
				array( $this, 'render_quality_level_original' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-general',   // Section
				array()   // Arguments for Renderer Function
			);

		}, 12 );

		//////////////////////////
		// Add some Setting Fields
		// * Advanced Settings Section
		add_action( 'admin_init', function() {

			// Guarantee a Secure Filename
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-safe-name',   // Field ID (Slug)
				__( 'Guarantee a Secure Filename', 'stillbedomain' ),   // Field Label
				array( $this, 'render_toggle_options' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array(   // Arguments for Renderer Function
					'field'       => 'safe-name',
					'description' => __( 'Convert the filename to a safe name for URL when its name contains anything other than half-width alphanumeric characters and -_.', 'stillbedomain' ),
				)
			);

			// Autoset Alt from Exif
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-autoset-alt',   // Field ID (Slug)
				__( 'Autoset Alt from Exif', 'stillbedomain' ),   // Field Label
				array( $this, 'render_toggle_options' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array(   // Arguments for Renderer Function
					'field'       => 'autoset-alt',
					'description' => __( 'Alt can be set automatically from the EXIF of the image when uploading.', 'stillbedomain' ),
				)
			);

			// Add a Suffix to Indicate Quality Level
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-level-suffix',   // Field ID (Slug)
				__( 'Add a Quality Level Suffix', 'stillbedomain' ),   // Field Label
				array( $this, 'render_toggle_options' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array(   // Arguments for Renderer Function
					'field'       => 'level-suffix',
					'description' => __( 'You can add a suffix to the resized file name to indicate the compression quality.', 'stillbedomain' ),
				)
			);

			// Enable Interlace
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-enable-interlace',   // Field ID (Slug)
				__( 'Enable Interlace/Progressive', 'stillbedomain' ),   // Field Label
				array( $this, 'render_toggle_options' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array(   // Arguments for Renderer Function
					'field'       => 'enable-interlace',
					'description' => array(
						__( 'Enable interlace (PNG) or progressive (JPEG) flag.', 'stillbedomain' ),
						__( 'If user&#39s network is unstable, the image will be displayed gradually from the low resolution.', 'stillbedomain' ),
						__( 'However, decoding requires the client to be about three times as heavy as disabling it.', 'stillbedomain' ). '<br>'.
						__( 'You may need to be careful when your site has an extremely large number of images.', 'stillbedomain' ),
						__( 'WebP does not offer a progressive or interlaced decoding refresh in the JPEG or PNG sense.', 'stillbedomain' ). '<br>'.
						sprintf( __( 'Refer: <a href="%1$s">%1$s</a>', 'stillbedomain' ), esc_url( 'https://developers.google.com/speed/webp/faq#does_webp_support_progressive_or_interlaced_display' ) ),
					),
				)
			);

			// Enable WebP
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-enable-webp',   // Field ID (Slug)
				__( 'Enable Generating WebP', 'stillbedomain' ),   // Field Label
				array( $this, 'render_toggle_options' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array(   // Arguments for Renderer Function
					'field'       => 'enable-webp',
					'description' => array(
						__( 'Generate WebP images & If user&#39s browser supports WebP format, WebP will be delivered preferentially.', 'stillbedomain' ),
						'<small class="your-server-status">'. __( 'WebP enablement on your server:', 'stillbedomain' ).
							( $this->_editor::_server_cmd_exists( 'cwebp' ) || $this->_editor::supports_mime_type( 'image/webp' ) ?
								'<em class="available">'. __( 'Available', 'stillbedomain' ) :
								'<em class="unavailable">'. __( 'Unavailable', 'stillbedomain' )
							).
						'</em></small>',
					),
				)
			);

			// Enable cwep Liberary
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-enable-cwebp',   // Field ID (Slug)
				__( 'Use cwebp Utility', 'stillbedomain' ),   // Field Label
				array( $this, 'render_toggle_options' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array(   // Arguments for Renderer Function
					'field'       => 'enable-cwebp',
					'description' => array(
						__( 'Use the &quot;cwebp&quot; utility when generating WebP.', 'stillbedomain' ). '<br>'.
						__( '(You need to have cwebp installed on your server. If not, it will not be used)', 'stillbedomain' ),
						__( 'Enabling cwebp gives you more options than generating WebP with Imagick or GD.', 'stillbedomain' ),
						'<small class="your-server-status">'. __( '&quot;cwebp&quot; enablement on your server:', 'stillbedomain' ).
							( $this->_editor::_server_cmd_exists( 'cwebp' ) ?
								'<em class="available">'. __( 'Available', 'stillbedomain' ) :
								'<em class="unavailable">'. __( 'Unavailable', 'stillbedomain' )
							).
						'</em><i>ver. '. $this->_editor::get_cwebp_version(). '</i></small>',
					),
				)
			);

			// Enable Lossless Compression
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-enable-webp-lossless',   // Field ID (Slug)
				__( 'Enable WebP Lossless Compression', 'stillbedomain' ),   // Field Label
				array( $this, 'render_toggle_options' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array(   // Arguments for Renderer Function
					'field'       => 'enable-webp-lossless',
					'description' => array(
						__( 'You can enable lossless compression when compressing PNG and GIF.', 'stillbedomain' ). '<br>'.
						__( 'If you feel that PNG compression is slow, disable this option.', 'stillbedomain' ),
						__( 'This option is only valid if &quot;cwebp&quot; is enabled.', 'stillbedomain' ),
						'<small class="your-server-status">'. __( 'Lossless compression enablement on your server:', 'stillbedomain' ).
							( $this->_editor::_server_cmd_exists( 'cwebp' ) ?
								'<em class="available">'. __( 'Available', 'stillbedomain' ) :
								'<em class="unavailable">'. __( 'Unavailable', 'stillbedomain' )
							).
						'</em></small>',
					),
				)
			);

			// Enable Near Lossless Compression
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-enable-webp-near-lossless',   // Field ID (Slug)
				__( 'Enable WebP Near-Lossless Compression', 'stillbedomain' ),   // Field Label
				array( $this, 'render_toggle_options' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array(   // Arguments for Renderer Function
					'field'       => 'enable-webp-near-lossless',
					'description' => array(
						__( 'You can enable near-lossless compression when compressing PNG and GIF.', 'stillbedomain' ). '<br>'.
						__( 'Near-Lossless allows for less size while maintaining quality comparable to lossless compression. (Lossy compression)', 'stillbedomain' ),
						__( 'This option is only valid if &quot;cwebp&quot; is enabled.', 'stillbedomain' ). '<br>'.
						__( 'The supported version is 0.5.0 or higher.', 'stillbedomain' ),
						'<small class="your-server-status">'. __( 'Near-Lossless compression enablement on your server:', 'stillbedomain' ).
							( $this->_editor::is_near_lossless() ?
								'<em class="available">'. __( 'Available', 'stillbedomain' ) :
								'<em class="unavailable">'. __( 'Unavailable', 'stillbedomain' )
							).
						'</em></small>',
					),
				)
			);

			// Reset Settings
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-reset-settings',   // Field ID (Slug)
				__( 'Reset Settings', 'stillbedomain' ),   // Field Label
				array( $this, 'render_reset_settings' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array()   // Arguments for Renderer Function
			);

			// Run Re-Compression
			add_settings_field(
				STILLBE_IQ_PREFIX. 'sf-recompression',   // Field ID (Slug)
				__( 'Run Re-Compression', 'stillbedomain' ),   // Field Label
				array( $this, 'render_recompression' ),   // Rederer
				STILLBE_IQ_PREFIX. 'setting-page',   // Rendering Page
				STILLBE_IQ_PREFIX. 'ss-advanced',   // Section
				array()   // Arguments for Renderer Function
			);

		}, 13 );

	}


	// Render Setting Form Wrapper & Common Style
	public function render_setting_page() {

		// Wrapper
		echo '<div class="wrap">';

			// Title
			echo '<h1>'. __( 'Quality Level Settings of Image Compression', 'stillbedomain' ). '</h1>';

			// RAW Data
			echo '<!-- '.json_encode( $this->settings ). ' -->';

			// Setting Form
			echo '<form name="img-quality-control-setting-form" method="POST" action="options.php">';

				// Since it will be sanitized twice at the time of initial setting, add a flag to skip the second sanitize
				//   * This is because the core function "update_option" is sanitized in it if option does not exist,
				//     and then "add_option" is executed and sanitized in it too.
				echo '<input type="hidden" name="'. self::SETTING_NAME. '[not-sanitized]" value="true">';

				// Add the information using Setting API
				// Group Name / Action Name / Nonce / This Page Path
				settings_fields( self::SETTING_GROUP );

				// Output All Sections
				do_settings_sections( STILLBE_IQ_PREFIX. 'setting-page' );

				// Submit Button
				submit_button();

			// Closing From
			echo '</form>';

		// Closing Wrapper
		echo '</div>';

	}


	// Check Month, date, hour, minute
	protected function chk_num_type( $value, $type = null ) {

		// Is Numeric?
		if( ! is_numeric( $value ) ) {
			return false;
		}

		// Cast the Value
		$_value = absint( $value );

		// Range
		switch( $type ) {
			case 'jpeg':
			case 'webp':
				$min = 1;
				$max = 100;
			break;
			case 'png':
				$min = 1;
				$max = 9;
			break;
			default:
				$min = 2;
				$max = 1;
		}

		// In Range?
		if( $min > $value || $max < $value ) {
			return false;
		}

		// Return the Casted Value
		return $_value;

	}


	// Sanitize Setting Values
	public function sanitize_setting( $input ) {

		// for Debug
	//	exit(json_encode($input));

		// Sanitized data is passed through
		if( empty( $input['not-sanitized'] ) ) {
			return $input;
		}

		// Save Data
		$save = array();

		// Quality Levels
		$qualities = $input['quality'] ?: array();
		$save['quality'] = array();
		foreach( $qualities as $size => $q ) {
			foreach( $q as $type => $v ) {
				$_tmp = $this->chk_num_type( $v, $type );
				if( false === $_tmp ) {
					continue;
				}
				$q_name = $size. '_'. $type;
				$save['quality'][ $q_name ] = $_tmp;
			}
		}

		// Do NOT Empty the Default Value
		if( empty( $save['quality']['default_jpeg'] ) ) {
			$save['quality']['default_jpeg'] = $this->_default['default_jpeg'];
		}
		if( empty( $save['quality']['default_png'] ) ) {
			$save['quality']['default_png']  = $this->_default['default_png'];
		}
		if( empty( $save['quality']['default_webp'] ) ) {
			$save['quality']['default_webp'] = $this->_default['default_webp'];
		}
		if( empty( $save['quality']['original_webp'] ) ) {
			$save['quality']['original_webp'] = $this->_default['original_webp'];
		}

		// Toggle Options
		$toggles = $input['toggle'] ?: array();
		$save['toggle'] = array();
		foreach( $toggles as $option => $bool ) {
			$save['toggle'][ $option ] = ( 'true' === $bool );
		}

		// If WebP is Enabled, Put ".htaccess" in the uploads Directory
		_stillbe_iqc_htaccess_webp( ! empty( $save['toggle']['enable-webp'] ) );

		// for Debug
	//	exit(json_encode($save));

		// Save!!
		return $save;

	}


	// Section Description; General Settings
	public function render_sd_general() {

		echo '<p>'. __( 'Set the quality level of image compression that are automatically resized when you upload to WP media library for each size.', 'stillbedomain' ). '</p>';

	}


	// Section Description; Advanced Setting
	public function render_sd_advanced() {

		echo '<p>'. __( 'Other advanced settings.', 'stillbedomain' ). '<br>';
		echo __( 'If you are not sure, do not change it unnecessarily.', 'stillbedomain' ). '</p>';

		// Donate
		echo '<p>'. __( 'With your donation, this plugin will be available for free forever.', 'stillbedomain' ). '<br>';
		echo __( 'If you like this plugin, please donate from the following.', 'stillbedomain' ). '</p>';

		// Load the PayPal Button
		include( STILLBE_IQ_BASE_DIR. '/asset/temp-donate-paypal.php' );

	}


	// Render Quality Level Table
	public function render_quality_level_table( $args ) {

		// Current Setting
		$qualities = $this->current['quality'];

		// Image Sizes
		$sizes = wp_get_registered_image_subsizes();
		uasort( $sizes, function( $a, $b ) {
			if( $a['width'] !== $b['width'] ) {
				return $a['width'] - $b['width'];
			}
			if( $a['height'] !== $b['height'] ) {
				return $a['height'] - $b['height'];
			}
			return (int) $a['crop'] - (int) $b['crop'];
		} );

		// Checked Mark SVG
		$checked = '<img src="data:image/svg+xml;base64,PHN2ZyBhcmlhLWhpZGRlbj0idHJ1ZSIgZm9jdXNhY'.
		           'mxlPSJmYWxzZSIgZGF0YS1wcmVmaXg9ImZhciIgZGF0YS1pY29uPSJjaGVjay1jaXJjbGUiIGNsYX'.
		           'NzPSJzdmctaW5saW5lLS1mYSBmYS1jaGVjay1jaXJjbGUgZmEtdy0xNiIgcm9sZT0iaW1nIiB4bWx'.
		           'ucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCA1MTIgNTEyIj48cGF0'.
		           'aCBmaWxsPSJjdXJyZW50Q29sb3IiIGQ9Ik0yNTYgOEMxMTkuMDMzIDggOCAxMTkuMDMzIDggMjU2c'.
		           'zExMS4wMzMgMjQ4IDI0OCAyNDggMjQ4LTExMS4wMzMgMjQ4LTI0OFMzOTIuOTY3IDggMjU2IDh6bT'.
		           'AgNDhjMTEwLjUzMiAwIDIwMCA4OS40NTEgMjAwIDIwMCAwIDExMC41MzItODkuNDUxIDIwMC0yMDA'.
		           'gMjAwLTExMC41MzIgMC0yMDAtODkuNDUxLTIwMC0yMDAgMC0xMTAuNTMyIDg5LjQ1MS0yMDAgMjAw'.
		           'LTIwMG0xNDAuMjA0IDEzMC4yNjdsLTIyLjUzNi0yMi43MThjLTQuNjY3LTQuNzA1LTEyLjI2NS00L'.
		           'jczNi0xNi45Ny0uMDY4TDIxNS4zNDYgMzAzLjY5N2wtNTkuNzkyLTYwLjI3N2MtNC42NjctNC43MD'.
		           'UtMTIuMjY1LTQuNzM2LTE2Ljk3LS4wNjlsLTIyLjcxOSAyMi41MzZjLTQuNzA1IDQuNjY3LTQuNzM'.
		           '2IDEyLjI2NS0uMDY4IDE2Ljk3MWw5MC43ODEgOTEuNTE2YzQuNjY3IDQuNzA1IDEyLjI2NSA0Ljcz'.
		           'NiAxNi45Ny4wNjhsMTcyLjU4OS0xNzEuMjA0YzQuNzA0LTQuNjY4IDQuNzM0LTEyLjI2Ni4wNjctM'.
		           'TYuOTcxeiI+PC9wYXRoPjwvc3ZnPg==" style="width: auto; height: 1.2em;">';

		// Render HTML
		echo '<p>'. __( 'Set the quality level for each size when WordPress resize.', 'stillbedomain' ). '<br>'. __( 'Blanks and 0 or less are not set.', 'stillbedomain' ). '</p>';
		echo '<p>'. __( 'Enter a number from 1-100 for JPEG and WebP, and a number from 1-9 for PNG.', 'stillbedomain' ). '</p>';
		echo '<p>';
		echo   __( 'For JPEG / WebP, about 80-85 is a value with a good balance between file size and image quality.', 'stillbedomain' ). ' ';
		echo   __( 'The lower quality level, the smaller the file size, but it lacks the image quality.', 'stillbedomain' );
		echo '</p>';
		echo '<p>';
		echo   __( 'For PNG, 9 is the best choice because it has the highest compression ratio.', 'stillbedomain' ). ' ';
		echo   __( 'Lowering the quality level does not change the image quality, but increases the file size.', 'stillbedomain' ). '<br>';
		echo   __( 'Try changing the level to 5-6 only if the server load for PNG resizing is high.', 'stillbedomain' ). '<br>';
		echo   __( 'Especially, large PNG images take a long time to losslessly compress WebP.', 'stillbedomain' ). ' ';
		echo   __( '(WebP lossless compression uses PNG quality level)', 'stillbedomain' );
		echo '</p>';
		echo '<div class="scroll-table-wrapper">';
		echo   '<table class="qulity-level-table">';
		echo     '<thead>';
		echo       '<tr>';
		echo         '<th>'. __( 'Size Name', 'stillbedomain' ). '</th>';
		echo         '<th>'. __( 'Max Width', 'stillbedomain' ). '</th>';
		echo         '<th>'. __( 'Max Height', 'stillbedomain' ). '</th>';
		echo         '<th>'. __( 'Cropping', 'stillbedomain' ). '</th>';
		echo         '<th>JPEG</th>';
		echo         '<th>PNG</th>';
		echo         '<th>WebP</th>';
		echo       '</tr>';
		echo     '<thead>';
		echo     '<tbody>';
		foreach( $sizes as $name => $size ) {
			$q_jpeg = empty( $qualities[( $name. '_jpeg' )] ) ? '' : $qualities[( $name. '_jpeg' )];
			$q_png  = empty( $qualities[( $name. '_png'  )] ) ? '' : $qualities[( $name. '_png'  )];
			$q_webp = empty( $qualities[( $name. '_webp' )] ) ? '' : $qualities[( $name. '_webp' )];
			$_class = 'area-'. ( (int) $size['width'] * (int) $size['height'] );
			echo   '<tr class="'. $_class. '">';
			echo     '<th>'. esc_html( $name ). '</th>';
			echo     '<th>'. esc_html( $size['width']  ). ( empty( $size['width']  ) ? '' : 'px' ). '</th>';
			echo     '<th>'. esc_html( $size['height'] ). ( empty( $size['height'] ) ? '' : 'px' ). '</th>';
			echo     '<th>'. ( empty( $size['crop'] ) ? '-' : $checked ). '</th>';
			echo     '<td><input type="number" name="'. self::SETTING_NAME. '[quality]['. esc_attr( $name ). '][jpeg]" value="'. esc_attr( $q_jpeg ). '"></td>';
			echo     '<td><input type="number" name="'. self::SETTING_NAME. '[quality]['. esc_attr( $name ). '][png]"  value="'. esc_attr( $q_png  ). '"></td>';
			echo     '<td><input type="number" name="'. self::SETTING_NAME. '[quality]['. esc_attr( $name ). '][webp]" value="'. esc_attr( $q_webp ). '"></td>';
			echo   '</tr>';
		}
		echo     '<tbody>';
		echo   '</table>';
		echo '</div>';

	}


	// Render Default Quality Level
	public function render_quality_level_default( $args ) {

		// Current Setting
		$qualities = $this->current['quality'];

		// Default Quality
		$def_jpeg = empty( $qualities['default_jpeg'] ) ? '' : $qualities['default_jpeg'];
		$def_png  = empty( $qualities['default_png']  ) ? '' : $qualities['default_png'] ;
		$def_webp = empty( $qualities['default_webp'] ) ? '' : $qualities['default_webp'];

		// Render HTML
		echo '<p>'. __( 'This quality levels are used when it is not set in the above table.', 'stillbedomain' ). '</p>';
		echo '<div class="scroll-table-wrapper">';
		echo   '<table class="qulity-level-table">';
		echo     '<thead>';
		echo       '<tr>';
		echo         '<th>JPEG</th>';
		echo         '<th>PNG</th>';
		echo         '<th>WebP</th>';
		echo       '</tr>';
		echo     '<thead>';
		echo     '<tbody>';
		echo       '<tr>';
		echo         '<td><input type="number" name="'. self::SETTING_NAME. '[quality][default][jpeg]" value="'. esc_attr( $def_jpeg ). '"></td>';
		echo         '<td><input type="number" name="'. self::SETTING_NAME. '[quality][default][png]"  value="'. esc_attr( $def_png  ). '"></td>';
		echo         '<td><input type="number" name="'. self::SETTING_NAME. '[quality][default][webp]" value="'. esc_attr( $def_webp ). '"></td>';
		echo       '</tr>';
		echo     '<tbody>';
		echo   '</table>';
		echo '</div>';

	}


	// Render Quality Level for WebP of the Original Image
	public function render_quality_level_original( $args ) {

		// Current Setting
		$qualities = $this->current['quality'];

		// Default Quality
		$original_webp = empty( $qualities['original_webp'] ) ? '' : $qualities['original_webp'];

		// Render HTML
		echo '<p>'. __( 'Set the quality level for WebP that is the same size as the uploaded original image.', 'stillbedomain' ). '</p>';
		echo '<div class="field-line">';
		echo   '<label>';
		echo     '<input type="number" name="'. self::SETTING_NAME. '[quality][original][webp]" value="'. esc_attr( $original_webp ). '">';
		echo   '</label>';
		echo '</div>';

	}


	// Render Re-Compression
	public function render_recompression() {

		// Nonce
		$nonces = array(
			'getIds'   => wp_create_nonce( 'sb-iqc-get-attachments' ),
			'reGenImg' => wp_create_nonce( 'sb-iqc-regenerate-images' ),
		);

		// Translate
		$translate = array(
			'An error has occurred.... Please try again.' => __( 'An error has occurred.... Please try again.', 'stillbedomain' ),
			'All the regeneration is done!'               => __( 'All the regeneration is done!',               'stillbedomain' ),
		);

		// Render HTML
		echo '<button type="button" id="get_attachment_id">'. __( 'Get a list of current image IDs', 'stillbedomain' ). '</button>';
		echo '<div id="ids_list"><div class="scroll-wrapper">'. __( 'Get the list of image IDs to be regenerated.', 'stillbedomain' ). '</div></div>';
		echo '<button type="button" id="regenerate_images">'. __( 'Regenerate resized images', 'stillbedomain' ). '</button>';
		echo '<script type="text/javascript">';
		echo   'window.$stillbe = (window.$stillbe || {});';
		echo   'window.$stillbe.admin = (window.$stillbe.admin || {});';
		echo   'window.$stillbe.admin.ajaxUrl = "'. esc_url( admin_url( 'admin-ajax.php' ) ). '";';
		echo   'window.$stillbe.admin.nonce = '. json_encode( $nonces ). ';';
		echo   'window.$stillbe.admin.translate = '. json_encode( $translate ). ';';
		echo '</script>';

	}


	// Advanced Toggle Options
	public function render_toggle_options( $args = array() ) {

		if( empty( $args['field'] ) ) {
			return;
		}

		// Field
		$field = $args['field'];

		// Current Setting
		$toggles = $this->current['toggle'] ?: array();
		$toggle  = ! empty( $toggles[ $field ] );

		// Input Name
		$name = self::SETTING_NAME. '[toggle]['. $field. ']';

		// Render HTML
		echo '<div class="field-line">';
		echo   '<label>';
		echo     '<input type="radio" name="'. $name. '" value="true"'.  ( $toggle ? ' checked' : '' ). '>';
		echo     '<span>'. __( 'Enable', 'stillbedomain' ). '</span>';
		echo   '</label>';
		echo   '<label>';
		echo     '<input type="radio" name="'. $name. '" value="false"'. ( $toggle ? '' : ' checked' ). '>';
		echo     '<span>'. __( 'Disable', 'stillbedomain' ). '</span>';
		echo   '</label>';
		echo '</div>';
		if( ! empty( $args['description'] ) ) {
			foreach( (array) $args['description'] as $d ) {
				echo '<p>'. $d. '</p>';
			}
		}

	}


	// Reset Settings
	public function render_reset_settings( $args ) {

		// Render HTML
		echo '<button type="button" id="reset_settings">'. __( 'Restore to default value', 'stillbedomain' ). '</button>';
		echo '<script type="text/javascript">';
		echo   '(function(){';
		echo     'const button = document.getElementById("reset_settings");';
		echo     'if(!button){';
		echo       'return null;';
		echo     '}';
		echo     'const xhr = new XMLHttpRequest();';
		echo     'const dat = new FormData();';
		echo     'let refreshing = false;';
		echo     'xhr.open("POST", "'. esc_url( admin_url( 'admin-ajax.php' ) ). '", true);';
		echo     'xhr.onload = function(){';
		echo       'let result = false;';
		echo       'try{';
		echo         'result = JSON.parse(xhr.responseText);';
		echo       '} catch{}';
		echo       'if(!result || !("ok" in (result || {}))){';
		echo         'alert("'. __( 'An error has occurred.... Please try again.', 'stillbedomain' ). '");';
		echo         'return false;';
		echo       '}';
		echo       'console.log(result);';
		echo       'alert(result.message);';
		echo       'if(result.ok){';
		echo         'location.reload();';
		echo       '}';
		echo       'refreshing = false;';
		echo     '};';
		echo     'xhr.onerror = function(){';
		echo       'alert("'. __( 'An error has occurred.... Please try again.', 'stillbedomain' ). '");';
		echo       'refreshing = false;';
		echo     '};';
		echo     'xhr.ontimeout = function(){';
		echo       'alert("'. __( 'An error has occurred.... Please try again.', 'stillbedomain' ). '");';
		echo       'refreshing = false;';
		echo     '};';
		echo     'dat.append("action", "sb_iqc_reset_settings");';
		echo     'dat.append("_nonce", "'. wp_create_nonce( 'sb-iqc-reset-settings' ). '");';
		echo     'button.onclick = function(){';
		echo       'if(refreshing){';
		echo         'return null;';
		echo       '}';
		echo       'refreshing = true;';
		echo       'xhr.send(dat);';
		echo     '};';
		echo   '})();';
		echo '</script>';

	}


}