<?php


/**
 * GD を使った Image Editor を継承した class
 * 
 */


// Do not allow direct access to the file.
if( ! defined( 'ABSPATH' ) ) {
	exit;
}




class StillBE_WP_Image_Editor_GD extends WP_Image_Editor_GD {

	use StillBE_Image_Editor_Common;

	const IMAGE_EDITOR_LIBRARY = 'gd';

	// _save メソッドで PNG の圧縮品質を指定できないので上書きする
	// ついでにインターレースを有効にする
	protected function _save( $image, $filename = null, $mime_type = null ) {
		list( $filename, $extension, $mime_type ) = $this->get_output_format( $filename, $mime_type );
		if( ! $filename ) {
			$filename = $this->generate_filename( null, null, $extension );
		}
		// Enable interlace
		if( function_exists( 'imageinterlace' ) && apply_filters( 'stillbe_image_quality_control_enable_interlace', STILLBE_IQ_ENABLE_INTERLACE ) ) {
			imageinterlace( $image, true );
		}
		if( 'image/webp' === $mime_type ) {
			// WebP
			if( ! $this->make_image( $filename, 'imagewebp', array( $image, $filename, $this->get_quality() ) ) ) {
				return new WP_Error( 'image_save_error', __( 'Image Editor Save Failed' ) );
			}
		} elseif( 'image/png' === $mime_type ) {
			// PNG
			// Convert from full colors to index colors, like original PNG.
			if( function_exists( 'imageistruecolor' ) && ! imageistruecolor( $image ) ) {
				imagetruecolortopalette( $image, false, imagecolorstotal( $image ) );
			}
			if( ! $this->make_image( $filename, 'imagepng', array( $image, $filename, $this->get_quality() ) ) ) {
				return new WP_Error( 'image_save_error', __( 'Image Editor Save Failed' ) );
			}
		} else{
			// Others
			return parent::_save( $image, $filename, $mime_type );
		}
		// Set correct file permissions.
		$stat  = stat( dirname( $filename ) );
		$perms = $stat['mode'] & 0000666; // Same permissions as parent folder, strip off the executable bits.
		chmod( $filename, $perms );
		return array(
			'path'      => $filename,
			'file'      => wp_basename( apply_filters( 'image_make_intermediate_size', $filename ) ),
			'width'     => $this->size['width'],
			'height'    => $this->size['height'],
			'mime-type' => $mime_type,
		);
	}

	// 
	public static function supports_mime_type( $mime_type ) {
		if( 'image/webp' === $mime_type ) {
			$image_types = imagetypes();
			return ( $image_types & IMG_WEBP ) != 0;
		}
		return parent::supports_mime_type( $mime_type );
	}

	// 
	protected function _make_webp_embed_library( $filename = null, $size_data = array() ) {
		// Check Filename
		if( empty( $filename ) ) {
			return new WP_Error( 'error_webp_filename', __( 'No WebP filename has be passed' ) );
		}
		// Store Original Size
		$orig_size = $this->size;
		// Max Size
		if( ! isset( $size_data['width'] ) ) {
			$size_data['width'] = $this->size['width'];
		}
		if ( ! isset( $size_data['height'] ) ) {
			$size_data['height'] = $this->size['height'];
		}
		if ( ! isset( $size_data['crop'] ) ) {
			$size_data['crop'] = false;
		}
		// Get the Quality
		$current_quality = $this->get_quality();
		$size            = empty( $size_data['size_name'] ) ? 'default' : $size_data['size_name'];
		$webp_quality    = $this->get_quality_from_size( $size, 'image/webp' );
		$this->set_quality( $webp_quality );
		// Resize
		$resized = $this->_resize( $size_data['width'], $size_data['height'], $size_data['crop'] );
		// Save
		if( is_wp_error( $resized ) ) {
			$this->size = $orig_size;
			$this->set_quality( $current_quality );
			return $resized;
		} else {
			$saved = $this->_save( $resized, $filename, 'image/webp' );
			imagedestroy( $resized );
		}
		$this->size = $orig_size;
		$this->set_quality( $current_quality );
		$file_size  = @filesize( $saved['path'] );
		return array(
			'path' => $saved['path'],
			'file' => $saved['file'],
			'size' => $file_size,
		);
	}

}




// END of the File



