<?php


/**
 * Imagick を使った Image Editor を継承した class
 * 
 */


// Do not allow direct access to the file.
if( ! defined( 'ABSPATH' ) ) {
	exit;
}




class StillBE_WP_Image_Editor_Imagick extends WP_Image_Editor_Imagick {

	use StillBE_Image_Editor_Common;

	const IMAGE_EDITOR_LIBRARY = 'imagick';

	protected function _save( $image, $filename = null, $mime_type = null ) {
		// Set PNG Compression Level
		if( 'image/png' === $this->mime_type ) {
			$this->image->setOption( 'png:compression-level', (string) $this->get_quality() );
		}
		// Set Interlace
		if( method_exists( $this->image, 'setInterlaceScheme' ) && apply_filters( 'stillbe_image_quality_control_enable_interlace', STILLBE_IQ_ENABLE_INTERLACE ) ) {
			$this->image->setInterlaceScheme( Imagick::INTERLACE_PLANE );
		}
		// Call the Parent Class Method
		return parent::_save( $image, $filename, $mime_type );
	}

	// 
	protected function _make_webp_embed_library( $filename = null, $size_data = array() ) {
		// Check Filename
		if( empty( $filename ) ) {
			return new WP_Error( 'error_webp_filename', __( 'No WebP filename has be passed' ) );
		}
		// Store Original Size & Image
		$orig_size  = $this->size;
		$orig_image = $this->image->getImage();
		// Max Size
		if( ! isset( $size_data['width'] ) ) {
			$size_data['width'] = $this->size['width'];
		}
		if ( ! isset( $size_data['height'] ) ) {
			$size_data['height'] = $this->size['height'];
		}
		if ( ! isset( $size_data['crop'] ) ) {
			$size_data['crop'] = false;
		}
		// Get the Quality
		$current_quality = $this->get_quality();
		$size            = empty( $size_data['size_name'] ) ? 'default' : $size_data['size_name'];
		$webp_quality    = $this->get_quality_from_size( $size, 'image/webp' );
		$this->set_quality( $webp_quality );
		// Resize
		$resized = $this->resize( $size_data['width'], $size_data['height'], $size_data['crop'] );
		// Save
		if( is_wp_error( $resized ) ) {
			$this->size  = $orig_size;
			$this->image->clear();
			$this->image->destroy();
			$this->image = null;
			$this->image = $orig_image;
			$this->set_quality( $current_quality );
			return $resized;
		} else {
			$saved = $this->_save( $resized, $filename, 'image/webp' );
			$this->image->clear();
			$this->image->destroy();
			$this->image = null;
		}
		$this->size  = $orig_size;
		$this->image = $orig_image;
		$this->set_quality( $current_quality );
		$file_size   = @filesize( $saved['path'] );
		return array(
			'path' => $saved['path'],
			'file' => $saved['file'],
			'size' => $file_size,
		);
	}

}




// END of the File



