<?php


/**
 * コアの画像処理系に適応するフィルター
 * 
 */


// Do not allow direct access to the file.
if( ! defined( 'ABSPATH' ) ) {
	exit;
}




// 共通メソッドを定義
trait StillBE_Image_Editor_Common {

	// 圧縮品質一覧用の変数
	protected $qualities = null;

	// Vars related "cwebp"
	protected $var_cwebp = array();

	// サイズに対応した圧縮品質を取得する
	public function get_quality_from_size( $size, $mime_type = 'image/jpeg' ) {
		if( empty( $this->qualities ) ) {
			// Quality Setting Data
			$qualities = $this->_get_quality_array();
			$this->qualities = apply_filters( 'stillbe_image_quality_list', $qualities );
		}
		$mime    = explode( '/', $mime_type );
		$mime    = end( $mime );
		$name    = $size. '_'. $mime;
		$defname = 'default_'. $mime;
		$quality = empty( $this->qualities[ $name ] ) ?
		             apply_filters( "stillbe_image_quality_default_{$mime}", $this->qualities[ $defname ] ) :
		             $this->qualities[ $name ];
		$quality = absint( apply_filters( "stillbe_image_quality_{$size}_{$mime}", $quality ) );
		if( 'image/png' !== $mime_type && $quality > 100 ) {
			$quality = 100;
		}
		if( 'image/png' === $mime_type && $quality > 9 ){
			$quality = 9;
		}
		if( $quality < 1 ) {
			$quality = 1;
		}
		return $quality;
	}

	protected function _get_quality_array() {
		return apply_filters(
			'stillbe_image_quality_default_list',
			_stillbe_get_quality_level_array()
		);
	}

	// 半角英数字と-_以外が含まれる場合はファイル名を置換する
	public function generate_safe_filename( $filename = '', $suffix = null, $original = false ) {
		if( apply_filters( 'stillbe_image_quality_control_convert_safename', true ) &&
		      preg_match( '%(.*?/)([^/]*?[^/](?=[^\/\.\-_a-zA-Z0-9])[^/]*?)(\.(?:jpe?g|png|gif)(?:\.webp)?)$%', $filename, $m ) ) {
			// $m[1]: dir path, $m[2]: unsafe file name, $m[3]: file extension
			$now  = date_i18n( 'YmdHis' );
			$hash = substr( md5( $m[2] ), 7, 8 );
			if( $original ) {
				return $m[1]. "{$now}-{$hash}". $m[3];
			}
			// $suffix will be appended to the destination filename, just before the extension.
			if( ! $suffix ) {
				$suffix = $this->get_suffix();
			}
			return $m[1]. "{$now}-{$hash}-{$suffix}". $m[3];
		}
		return $filename;
	}


	// WP_Image_Editor の generate_filename を改造
	// 半角英数字と-と_以外が含まれている場合は安全な名前に変更する
	// 'stillbe_image_quality_control_convert_safename' フックに false を返すとファイル名を変更しない
	public function generate_filename( $suffix = null, $dest_path = null, $extension = null ) {
		// WP組込Classによる画像ファイル名
		$wp_generated_name = parent::generate_filename( $suffix, $dest_path, $extension );
		// 半角英数字と-_以外が含まれる場合はファイル名を置換する
		$wp_generated_name = $this->generate_safe_filename( $wp_generated_name, $suffix );
		// Return after Filtering
		return apply_filters( "stillbe_uploaded_image_{$this->mime_type}_name", $wp_generated_name );
	}

	// Suffix に Quality の値を追加する
	// ユーザが追加できる suffix 用のフィルターも追加する
	public function get_suffix() {
		$suffix = parent::get_suffix();
		if( false === $suffix ) {
			return false;
		}
		$user_additional_suffix = apply_filters( 'stillbe_image_quality_control_suffix_options', '' );
		$user_additional_suffix = apply_filters( 'stillbe_image_quality_control_suffix_options_'. self::IMAGE_EDITOR_LIBRARY, $user_additional_suffix );
		if( apply_filters( 'stillbe_image_quality_control_suffix_q_value', STILLBE_IQ_ENABLE_QUALITY_VALUE_SUFFIX ) ) {
			$quality = $this->get_quality();
			return $suffix. '-q'. $quality. $user_additional_suffix;
		}
		return $suffix. $user_additional_suffix;
	}

	public function make_subsize( $size_data ) {
		// 現在の設定を退避しておく
		$current_quality = $this->get_quality();
		// 圧縮品質を変更する
		$size    = empty( $size_data['size_name'] ) ? 'default' : $size_data['size_name'];
		$quality = $this->get_quality_from_size( $size, $this->mime_type );
		$this->set_quality( $quality );
		// リサイズして保存する
		$meta = parent::make_subsize( $size_data );
		if( ! is_wp_error( $meta ) ) {
			// Save WebP
		//	$uploads   = wp_upload_dir();
			$basedir   = dirname( $this->file );   // $uploads['path']
			$webp_name = apply_filters( "stillbe_uploaded_image_webp_name", "{$basedir}/{$meta['file']}.webp" );
			$webp_data = $this->make_webp( $webp_name, $size_data );
			if( is_wp_error( $webp_data ) || empty( $webp_data['size'] ) ) {
				$this->set_quality( $current_quality );
				return $meta;
			}
			$meta['file-webp'] = $webp_data['file'];
		}
		$this->set_quality( $current_quality );
		return $meta;
	}

	// Checks if the server command exists
	public static function _server_cmd_exists( $cmd = 'yf4us44q8u5u8xxxdmja' ) {   // Default Value is an Impossible Command
		$cmd = strval( $cmd );
		if( empty( $cmd ) ) {
			return false;
		}
		$man = shell_exec( sprintf( 'man -k %s', escapeshellarg( $cmd ) ) );
		return ! empty( $man );
	}

	// Convert to WebP with cwebp
	protected function _make_cwebp( $filename = null, $size_data = array() ) {
		// Check Filename
		if( empty( $filename ) ) {
			return new WP_Error( 'error_webp_filename', __( 'No WebP filename has be passed' ) );
		}
		// Check "-near_lossless" Option
		// Default is FALSE
		$is_near_lossless = false;
		if( apply_filters( 'stillbe_image_quality_control_enable_webp_near_lossless', STILLBE_IQ_ENABLE_WEBP_NEAR_LOSSLESS ) ) {
			$is_near_lossless = $this->is_near_lossless();
		}
		// Get the Quality
		$size         = empty( $size_data['size_name'] ) ? 'default' : $size_data['size_name'];
		$quality      = $this->get_quality_from_size( $size, $this->mime_type );
		$webp_quality = $this->get_quality_from_size( $size, 'image/webp' );
		// Set Compression Quality Options
		$options = '';
		if( 'image/png' === $this->mime_type || 'image/gif' === $this->mime_type ) {
			if( apply_filters( 'stillbe_image_quality_control_enable_webp_lossless_for_png_gif', STILLBE_IQ_ENABLE_WEBP_LOSSLESS ) ) {
				// LossLess Compression
				if( $is_near_lossless ) {
					$options .= "-near_lossless {$webp_quality}";
				} else {
					if( 'image/gif' === $this->mime_type ) {
						$quality = 6;
					}
					$quality  = $quality * 10 + 10;
					$options .= "-lossless -q {$quality}";
					// "-z" option is not supported at my rental server
				//	$options .= "-lossless -z {$quality}";
				}
			} else {
				// Lossy Compression
				$options .= "-q {$webp_quality}";
			}
		} elseif( 'image/jpeg' === $this->mime_type ) {
			// Lossy Compression
			$options .= "-q {$webp_quality}";
		}
		// Set Crop & Resiez Options
		if( ! isset( $size_data['width'] ) ) {
			$size_data['width'] = $this->size['width'];
		}
		if ( ! isset( $size_data['height'] ) ) {
			$size_data['height'] = $this->size['height'];
		}
		if ( ! isset( $size_data['crop'] ) ) {
			$size_data['crop'] = false;
		}
		if( $this->size['width'] > $size_data['width'] || $this->size['height'] > $size_data['height'] ) {
			$dims = image_resize_dimensions( $this->size['width'], $this->size['height'], $size_data['width'], $size_data['height'], $size_data['crop'] );
			if( ! $dims ) {
				return new WP_Error( 'error_getting_dimensions', __( 'Could not calculate resized image dimensions' ) );
			}
			list( $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h ) = $dims;
			if( $size_data['crop'] ) {
				$options .= " -crop {$src_x} {$src_y} {$src_w} {$src_h}";
			}
			$options .= " -resize {$dst_w} {$dst_h}";
		}
		// Convert Image with cwebp
		exec( sprintf( 'cwebp %s %s -o %s 2>&1', $options, $this->file, $filename ), $output, $result_code );
		if( $result_code > 0 ) {
			return new WP_Error( 'error_cwebp_exec', implode( "\n", $output ), $options );
		}
		if( ! file_exists( $filename ) ) {
			return new WP_Error( 'error_cwebp_exec', sprintf( __( '%s; File does not exist', 'stillbedomain' ), $filename ) );
		}
		// Return Result
		$file_size = @filesize( $filename );
		return array(
			'path' => $filename,
			'file' => wp_basename( $filename ),
			'size' => $file_size,
		);
	}

	// Convert to WebP with GD or Imagick
	abstract protected function _make_webp_embed_library( $filename, $size_data );

	// Generate WebP
	public function make_webp( $filename = null, $size_data = null ) {
		if( ! apply_filters( 'stillbe_image_quality_control_enable_webp', STILLBE_IQ_ENABLE_WEBP, 'generate' ) ) {
			return false;
		}
		$this->var_cwebp['is_exists'] = isset( $this->var_cwebp['is_exists'] ) ? $this->var_cwebp['is_exists'] : $this->_server_cmd_exists( 'cwebp' );
		if( apply_filters( 'stillbe_image_quality_control_enable_cwebp_lib', STILLBE_IQ_ENABLE_CWEBP_LIBRARY ) && $this->var_cwebp['is_exists'] ) {
			return $this->_make_cwebp( $filename, $size_data );
		} elseif( $this->supports_mime_type( 'image/webp' ) ) {
			return $this->_make_webp_embed_library( $filename, $size_data );
		}
		return false;
	}

	// Get the Version of cwebp
	public static function get_cwebp_version() {
		// Get the "cwebp" Version
		return trim( shell_exec( 'cwebp -version' ) );
	}

	// Get the Availability of the Near-Lossless Option
	public static function is_near_lossless() {
		// Get the "cwebp" Version
		$version = self::get_cwebp_version();
		// Version Check
		return version_compare( $version, '0.5.0', '>=' );
	}

}




// END of the File



